(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('react')) :
  typeof define === 'function' && define.amd ? define(['react'], factory) :
  (global = global || self, global.UseDownshift = factory(global.React));
}(this, function (react) { 'use strict';

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function _objectSpread(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};
      var ownKeys = Object.keys(source);

      if (typeof Object.getOwnPropertySymbols === 'function') {
        ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
          return Object.getOwnPropertyDescriptor(source, sym).enumerable;
        }));
      }

      ownKeys.forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    }

    return target;
  }

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  function _objectWithoutProperties(source, excluded) {
    if (source == null) return {};

    var target = _objectWithoutPropertiesLoose(source, excluded);

    var key, i;

    if (Object.getOwnPropertySymbols) {
      var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

      for (i = 0; i < sourceSymbolKeys.length; i++) {
        key = sourceSymbolKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
        target[key] = source[key];
      }
    }

    return target;
  }

  function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
  }

  function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
  }

  function _iterableToArrayLimit(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"] != null) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
  }

  var statusDiv = typeof document === 'undefined' ? null : document.getElementById('a11y-status-message');
  var cleanupTimerID;

  function setA11yStatus(status) {
    var div = getStatusDiv();

    if (!status) {
      return;
    }

    if (cleanupTimerID) {
      clearTimeout(cleanupTimerID);
      cleanupTimerID = null;
    }

    div.textContent = status;
    cleanupTimerID = setTimeout(function () {
      div.textContent = '';
      cleanupTimerID = null;
    }, 500);
  }

  function getStatusDiv() {
    if (statusDiv) {
      return statusDiv;
    }

    statusDiv = document.createElement('div');
    statusDiv.setAttribute('id', 'a11y-status-message');
    statusDiv.setAttribute('role', 'status');
    statusDiv.setAttribute('aria-live', 'polite');
    statusDiv.setAttribute('aria-relevant', 'additions text');
    Object.assign(statusDiv.style, {
      border: '0',
      clip: 'rect(0 0 0 0)',
      height: '1px',
      margin: '-1px',
      overflow: 'hidden',
      padding: '0',
      position: 'absolute',
      width: '1px'
    });
    document.body.appendChild(statusDiv);
    return statusDiv;
  }

  function usePrevious(value) {
    var ref = react.useRef();
    react.useEffect(function () {
      ref.current = value;
    });
    return ref.current;
  }

  function isElement(el) {
    return el != null && typeof el === 'object' && el.nodeType === 1;
  }

  function canOverflow(overflow, skipOverflowHiddenElements) {
    if (skipOverflowHiddenElements && overflow === 'hidden') {
      return false;
    }

    return overflow !== 'visible' && overflow !== 'clip';
  }

  function isScrollable(el, skipOverflowHiddenElements) {
    if (el.clientHeight < el.scrollHeight || el.clientWidth < el.scrollWidth) {
      var style = getComputedStyle(el, null);
      return canOverflow(style.overflowY, skipOverflowHiddenElements) || canOverflow(style.overflowX, skipOverflowHiddenElements);
    }

    return false;
  }

  function alignNearest(scrollingEdgeStart, scrollingEdgeEnd, scrollingSize, scrollingBorderStart, scrollingBorderEnd, elementEdgeStart, elementEdgeEnd, elementSize) {
    if (elementEdgeStart < scrollingEdgeStart && elementEdgeEnd > scrollingEdgeEnd || elementEdgeStart > scrollingEdgeStart && elementEdgeEnd < scrollingEdgeEnd) {
      return 0;
    }

    if (elementEdgeStart <= scrollingEdgeStart && elementSize <= scrollingSize || elementEdgeEnd >= scrollingEdgeEnd && elementSize >= scrollingSize) {
      return elementEdgeStart - scrollingEdgeStart - scrollingBorderStart;
    }

    if (elementEdgeEnd > scrollingEdgeEnd && elementSize < scrollingSize || elementEdgeStart < scrollingEdgeStart && elementSize > scrollingSize) {
      return elementEdgeEnd - scrollingEdgeEnd + scrollingBorderEnd;
    }

    return 0;
  }

  var computeScrollIntoView = (function (target, options) {
    var scrollMode = options.scrollMode,
        block = options.block,
        inline = options.inline,
        boundary = options.boundary,
        skipOverflowHiddenElements = options.skipOverflowHiddenElements;
    var checkBoundary = typeof boundary === 'function' ? boundary : function (node) {
      return node !== boundary;
    };

    if (!isElement(target)) {
      throw new TypeError('Invalid target');
    }

    var scrollingElement = document.scrollingElement || document.documentElement;
    var frames = [];
    var cursor = target;

    while (isElement(cursor) && checkBoundary(cursor)) {
      cursor = cursor.parentNode;

      if (cursor === scrollingElement) {
        frames.push(cursor);
        break;
      }

      if (cursor === document.body && isScrollable(cursor) && !isScrollable(document.documentElement)) {
        continue;
      }

      if (isScrollable(cursor, skipOverflowHiddenElements)) {
        frames.push(cursor);
      }
    }

    var viewportWidth = window.visualViewport ? visualViewport.width : innerWidth;
    var viewportHeight = window.visualViewport ? visualViewport.height : innerHeight;
    var viewportX = window.scrollX || pageXOffset;
    var viewportY = window.scrollY || pageYOffset;

    var _target$getBoundingCl = target.getBoundingClientRect(),
        targetHeight = _target$getBoundingCl.height,
        targetWidth = _target$getBoundingCl.width,
        targetTop = _target$getBoundingCl.top,
        targetRight = _target$getBoundingCl.right,
        targetBottom = _target$getBoundingCl.bottom,
        targetLeft = _target$getBoundingCl.left;

    var targetBlock = block === 'start' || block === 'nearest' ? targetTop : block === 'end' ? targetBottom : targetTop + targetHeight / 2;
    var targetInline = inline === 'center' ? targetLeft + targetWidth / 2 : inline === 'end' ? targetRight : targetLeft;
    var computations = [];

    for (var index = 0; index < frames.length; index++) {
      var frame = frames[index];

      var _frame$getBoundingCli = frame.getBoundingClientRect(),
          _height = _frame$getBoundingCli.height,
          _width = _frame$getBoundingCli.width,
          _top = _frame$getBoundingCli.top,
          right = _frame$getBoundingCli.right,
          bottom = _frame$getBoundingCli.bottom,
          _left = _frame$getBoundingCli.left;

      if (scrollMode === 'if-needed' && targetTop >= 0 && targetLeft >= 0 && targetBottom <= viewportHeight && targetRight <= viewportWidth && targetTop >= _top && targetBottom <= bottom && targetLeft >= _left && targetRight <= right) {
        return computations;
      }

      var frameStyle = getComputedStyle(frame);
      var borderLeft = parseInt(frameStyle.borderLeftWidth, 10);
      var borderTop = parseInt(frameStyle.borderTopWidth, 10);
      var borderRight = parseInt(frameStyle.borderRightWidth, 10);
      var borderBottom = parseInt(frameStyle.borderBottomWidth, 10);
      var blockScroll = 0;
      var inlineScroll = 0;
      var scrollbarWidth = 'offsetWidth' in frame ? frame.offsetWidth - frame.clientWidth - borderLeft - borderRight : 0;
      var scrollbarHeight = 'offsetHeight' in frame ? frame.offsetHeight - frame.clientHeight - borderTop - borderBottom : 0;

      if (scrollingElement === frame) {
        if (block === 'start') {
          blockScroll = targetBlock;
        } else if (block === 'end') {
          blockScroll = targetBlock - viewportHeight;
        } else if (block === 'nearest') {
          blockScroll = alignNearest(viewportY, viewportY + viewportHeight, viewportHeight, borderTop, borderBottom, viewportY + targetBlock, viewportY + targetBlock + targetHeight, targetHeight);
        } else {
          blockScroll = targetBlock - viewportHeight / 2;
        }

        if (inline === 'start') {
          inlineScroll = targetInline;
        } else if (inline === 'center') {
          inlineScroll = targetInline - viewportWidth / 2;
        } else if (inline === 'end') {
          inlineScroll = targetInline - viewportWidth;
        } else {
          inlineScroll = alignNearest(viewportX, viewportX + viewportWidth, viewportWidth, borderLeft, borderRight, viewportX + targetInline, viewportX + targetInline + targetWidth, targetWidth);
        }

        blockScroll = Math.max(0, blockScroll + viewportY);
        inlineScroll = Math.max(0, inlineScroll + viewportX);
      } else {
        if (block === 'start') {
          blockScroll = targetBlock - _top - borderTop;
        } else if (block === 'end') {
          blockScroll = targetBlock - bottom + borderBottom + scrollbarHeight;
        } else if (block === 'nearest') {
          blockScroll = alignNearest(_top, bottom, _height, borderTop, borderBottom + scrollbarHeight, targetBlock, targetBlock + targetHeight, targetHeight);
        } else {
          blockScroll = targetBlock - (_top + _height / 2) + scrollbarHeight / 2;
        }

        if (inline === 'start') {
          inlineScroll = targetInline - _left - borderLeft;
        } else if (inline === 'center') {
          inlineScroll = targetInline - (_left + _width / 2) + scrollbarWidth / 2;
        } else if (inline === 'end') {
          inlineScroll = targetInline - right + borderRight + scrollbarWidth;
        } else {
          inlineScroll = alignNearest(_left, right, _width, borderLeft, borderRight + scrollbarWidth, targetInline, targetInline + targetWidth, targetWidth);
        }

        var scrollLeft = frame.scrollLeft,
            scrollTop = frame.scrollTop;
        blockScroll = Math.max(0, Math.min(scrollTop + blockScroll, frame.scrollHeight - _height + scrollbarHeight));
        inlineScroll = Math.max(0, Math.min(scrollLeft + inlineScroll, frame.scrollWidth - _width + scrollbarWidth));
        targetBlock += scrollTop - blockScroll;
        targetInline += scrollLeft - inlineScroll;
      }

      computations.push({
        el: frame,
        top: blockScroll,
        left: inlineScroll
      });
    }

    return computations;
  });

  var idCounter = 0;
  var stateKeys = ['highlightedIndex', 'inputValue', 'isOpen', 'selectedItem', 'type'];
  function noop() {}
  function generateId() {
    return String(idCounter++);
  }
  function isOrContainsNode(parent, child) {
    return parent === child || parent.contains && parent.contains(child);
  }
  function scrollIntoView(node, menuNode) {
    if (node === null) {
      return;
    }

    var actions = computeScrollIntoView(node, {
      boundary: menuNode,
      block: 'nearest',
      scrollMode: 'if-needed'
    });
    actions.forEach(function (_ref) {
      var el = _ref.el,
          top = _ref.top,
          left = _ref.left;
      el.scrollTop = top;
      el.scrollLeft = left;
    });
  }
  function cbToCb(cb) {
    return typeof cb === 'function' ? cb : noop;
  }
  function getA11yStatusMessage(_ref2) {
    var isOpen = _ref2.isOpen,
        selectedItem = _ref2.selectedItem,
        resultCount = _ref2.resultCount,
        previousResultCount = _ref2.previousResultCount,
        itemToString = _ref2.itemToString;

    if (!isOpen) {
      return selectedItem ? itemToString(selectedItem) : '';
    }

    if (!resultCount) {
      return 'No results are available.';
    }

    if (resultCount !== previousResultCount) {
      return "".concat(resultCount, " result").concat(resultCount === 1 ? ' is' : 's are', " available, use up and down arrow keys to navigate. Press Enter key to select.");
    }

    return '';
  }
  function debounce(fn, time) {
    var timeoutId;

    function cancel() {
      if (timeoutId) {
        clearTimeout(timeoutId);
      }
    }

    function wrapper() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      cancel();
      timeoutId = setTimeout(function () {
        timeoutId = null;
        fn.apply(void 0, args);
      }, time);
    }

    wrapper.cancel = cancel;
    return wrapper;
  }
  function callAll() {
    for (var _len2 = arguments.length, fns = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      fns[_key2] = arguments[_key2];
    }

    return function () {
      for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      fns.forEach(function (fn) {
        if (fn) {
          fn.apply(void 0, args);
        }
      });
    };
  }
  function callAllEventHandlers() {
    for (var _len4 = arguments.length, fns = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      fns[_key4] = arguments[_key4];
    }

    return function (event) {
      for (var _len5 = arguments.length, args = new Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
        args[_key5 - 1] = arguments[_key5];
      }

      return fns.some(function (fn) {
        if (fn) {
          fn.apply(void 0, [event].concat(args));
        }

        return event.preventDownshiftDefault || event.hasOwnProperty('nativeEvent') && event.nativeEvent.preventDownshiftDefault;
      });
    };
  }
  function normalizeArrowKey(event) {
    var key = event.key,
        keyCode = event.keyCode;

    if (keyCode >= 37 && keyCode <= 40 && key.indexOf('Arrow') !== 0) {
      return "Arrow".concat(key);
    }

    return key;
  }
  function getNextWrappingIndex(moveAmount, baseIndex, itemCount) {
    var itemsLastIndex = itemCount - 1;

    if (typeof baseIndex !== 'number' || baseIndex < 0 || baseIndex >= itemCount) {
      baseIndex = moveAmount > 0 ? -1 : itemsLastIndex + 1;
    }

    var newIndex = baseIndex + moveAmount;

    if (newIndex < 0) {
      newIndex = itemsLastIndex;
    } else if (newIndex > itemsLastIndex) {
      newIndex = 0;
    }

    return newIndex;
  }
  function pickState() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var result = {};
    stateKeys.forEach(function (k) {
      if (state.hasOwnProperty(k)) {
        result[k] = state[k];
      }
    });
    return result;
  }

  var defaultParams = {
    defaultHighlightedIndex: null,
    defaultIsOpen: false,
    getA11yStatusMessage: getA11yStatusMessage,
    itemToString: function itemToString(i) {
      if (i == null) {
        return '';
      }

      return String(i);
    },
    onStateChange: noop,
    onInputValueChange: noop,
    onUserAction: noop,
    onChange: noop,
    onSelect: noop,
    onOuterClick: noop,
    selectedItemChanged: function selectedItemChanged(prevItem, item) {
      return prevItem !== item;
    },
    stateReducer: function stateReducer(_, stateToSet) {
      return stateToSet;
    },
    scrollIntoView: scrollIntoView
  };
  var initialState = {
    items: [],
    itemCount: null,
    isOpen: false,
    inputValue: '',
    selectedItem: null,
    highlightedIndex: null
  };

  function useDownshift(params) {
    params = _objectSpread({}, defaultParams, params);
    params.id = params.id || "use-downshift-".concat(generateId());
    params.menuId = params.menuId || "".concat(params.id, "-menu");
    params.labelId = params.labelId || "".concat(params.id, "-label");
    params.inputId = params.inputId || "".concat(params.id, "-input");

    params.getItemId = params.getItemId || function (index) {
      return "".concat(params.id, "-item-").concat(index);
    };

    var _useState = react.useState(_objectSpread({}, initialState, {
      isOpen: params.defaultIsOpen,
      highlightedIndex: params.defaultHighlightedIndex
    })),
        _useState2 = _slicedToArray(_useState, 2),
        state = _useState2[0],
        setState = _useState2[1];

    var previousState = usePrevious(state);
    var previousParams = usePrevious(params);
    var rootRef = react.useRef(null);
    var menuRef = react.useRef(null);
    var itemsRef = react.useRef([]);
    var itemCountRef = react.useRef(null);
    var avoidScrollingRef = react.useRef(null);
    var timeoutsIdsRef = react.useRef([]);
    var isMouseDownRef = react.useRef(false);
    var isTouchMoveRef = react.useRef(false);
    var previousResultCountRef = react.useRef(0);
    react.useEffect(function () {
      var targetWithinDownshift = function targetWithinDownshift(target) {
        var checkActiveElement = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
        return [rootRef.current, menuRef.current].some(function (contextNode) {
          return contextNode && (isOrContainsNode(contextNode, target) || checkActiveElement && isOrContainsNode(contextNode, window.document.activeElement));
        });
      };

      var onMouseDown = function onMouseDown() {
        isMouseDownRef.current = true;
      };

      var onMouseUp = function onMouseUp(event) {
        isMouseDownRef.current = false;
        var contextWithinDownshift = targetWithinDownshift(event.target);

        if (!contextWithinDownshift && getState().isOpen) {
          reset({
            type: useDownshift.stateChangeTypes.mouseUp
          });
          params.onOuterClick(getStateAndHelpers());
        }
      };

      var onTouchStart = function onTouchStart() {
        isTouchMoveRef.current = false;
      };

      var onTouchMove = function onTouchMove() {
        isTouchMoveRef.current = true;
      };

      var onTouchEnd = function onTouchEnd(event) {
        var contextWithinDownshift = targetWithinDownshift(event.target, false);

        if (!isTouchMoveRef.current && !contextWithinDownshift && getState().isOpen) {
          reset({
            type: useDownshift.stateChangeTypes.touchEnd
          });
          params.onOuterClick(getStateAndHelpers());
        }
      };

      window.addEventListener('mouseup', onMouseUp);
      window.addEventListener('mousedown', onMouseDown);
      window.addEventListener('touchstart', onTouchStart);
      window.addEventListener('touchmove', onTouchMove);
      window.addEventListener('touchend', onTouchEnd);
      return function () {
        internalClearTimeouts();
        window.removeEventListener('mouseup', onMouseUp);
        window.removeEventListener('mousedown', onMouseDown);
        window.removeEventListener('touchstart', onTouchStart);
        window.removeEventListener('touchmove', onTouchMove);
        window.removeEventListener('touchend', onTouchEnd);
      };
    }, []);
    react.useEffect(function () {
      if (isControlledParam('selectedItem') && params.selectedItemChanged(previousParams ? previousParams.selectedItem : null, params.selectedItem)) {
        internalSetState({
          type: useDownshift.stateChangeTypes.controlledPropUpdatedSelectedItem,
          inputValue: params.itemToString(params.selectedItem)
        });
      }

      if (!avoidScrollingRef && shouldScroll(previousState, previousParams)) {
        scrollHighlightedItemIntoView();
      }

      updateStatus();
    }, [state, params]);
    var updateStatus = debounce(function () {
      var state = getState();
      var item = itemsRef.current[state.highlightedIndex];
      var resultCount = getItemCount();
      var status = params.getA11yStatusMessage(_objectSpread({
        itemToString: params.itemToString,
        previousResultCount: previousResultCountRef.current,
        resultCount: resultCount,
        highlightedItem: item
      }, state));
      previousResultCountRef.current = resultCount;
      setA11yStatus(status);
    }, 200);

    var scrollHighlightedItemIntoView = function scrollHighlightedItemIntoView() {
      var node = getItemNodeFromIndex(getState().highlightedIndex);
      params.scrollIntoView(node, menuRef.current);
    };

    var shouldScroll = function shouldScroll(prevState, prevParams) {
      var _ref = params.highlightedIndex === undefined ? getState() : params,
          currentHighlightedIndex = _ref.highlightedIndex;

      var _ref2 = prevParams.highlightedIndex === undefined ? prevState : prevParams,
          prevHighlightedIndex = _ref2.highlightedIndex;

      var scrollWhenOpen = currentHighlightedIndex && getState().isOpen && !prevState.isOpen;
      var scrollWhenNavigating = currentHighlightedIndex !== prevHighlightedIndex;
      return scrollWhenOpen || scrollWhenNavigating;
    };

    var internalSetTimeout = function internalSetTimeout(fn, time) {
      var id = setTimeout(function () {
        timeoutsIdsRef.current = timeoutsIdsRef.current.filter(function (i) {
          return i !== id;
        });
        fn();
      }, time);
      timeoutsIdsRef.current.push(id);
    };

    var internalClearTimeouts = function internalClearTimeouts() {
      timeoutsIdsRef.current.forEach(function (id) {
        clearTimeout(id);
      });
      timeoutsIdsRef.current.length = 0;
    };

    var isControlledParam = function isControlledParam(key) {
      return params[key] !== undefined;
    };

    var getState = function getState() {
      var stateToMerge = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : state;
      return Object.keys(stateToMerge).reduce(function (s, key) {
        s[key] = isControlledParam(key) ? params[key] : stateToMerge[key];
        return s;
      }, {});
    };

    var internalSetState = function internalSetState(stateToSet, cb) {
      var isItemSelected, onChangeArg;
      var onStateChangeArg = {};
      var isStateToSetFunction = typeof stateToSet === 'function';

      if (!isStateToSetFunction && stateToSet.hasOwnProperty('inputValue')) {
        params.onInputValueChange(stateToSet.inputValue, _objectSpread({}, getStateAndHelpers(), stateToSet));
      }

      var s = getState(state);
      var newStateToSet = isStateToSetFunction ? stateToSet(s) : stateToSet;
      newStateToSet = params.stateReducer(s, newStateToSet);
      isItemSelected = newStateToSet.hasOwnProperty('selectedItem');
      var nextState = {};
      var nextFullState = {};

      if (isItemSelected && newStateToSet.selectedItem !== s.selectedItem) {
        onChangeArg = newStateToSet.selectedItem;
      }

      newStateToSet.type = newStateToSet.type || useDownshift.stateChangeTypes.unknown;
      Object.keys(newStateToSet).forEach(function (key) {
        if (s[key] !== newStateToSet[key]) {
          onStateChangeArg[key] = newStateToSet[key];
        }

        if (key === 'type') {
          return;
        }

        nextFullState[key] = newStateToSet[key];

        if (!isControlledParam(key)) {
          nextState[key] = newStateToSet[key];
        }
      });

      if (isStateToSetFunction && newStateToSet.hasOwnProperty('inputValue')) {
        params.onInputValueChange(newStateToSet.inputValue, _objectSpread({}, getStateAndHelpers(), newStateToSet));
      }

      setState(_objectSpread({}, state, nextState));
      internalSetTimeout(function () {
        cbToCb(cb)();
        var hasMoreStateThanType = Object.keys(onStateChangeArg).length > 1;

        if (hasMoreStateThanType) {
          params.onStateChange(onStateChangeArg, getStateAndHelpers());
        }

        if (isItemSelected) {
          params.onSelect(stateToSet.selectedItem, getStateAndHelpers());
        }

        if (onChangeArg !== undefined) {
          params.onChange(onChangeArg, getStateAndHelpers());
        }

        params.onUserAction(onStateChangeArg, getStateAndHelpers());
      });
    };

    var clearItems = function clearItems() {
      itemsRef.current.length = 0;
    };

    var reset = function reset() {
      var otherStateToSet = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var cb = arguments.length > 1 ? arguments[1] : undefined;
      otherStateToSet = pickState(otherStateToSet);
      internalSetState(function (_ref3) {
        var selectedItem = _ref3.selectedItem;
        return _objectSpread({
          isOpen: params.defaultIsOpen,
          highlightedIndex: params.defaultHighlightedIndex,
          inputValue: params.itemToString(selectedItem)
        }, otherStateToSet);
      }, cb);
    };

    var getItemNodeFromIndex = function getItemNodeFromIndex(index) {
      return window.document.getElementById(params.getItemId(index));
    };

    var getItemCount = function getItemCount() {
      var itemCount = itemsRef.current.length;

      if (itemCountRef.current != null) {
        itemCount = itemCountRef.current;
      } else if (params.itemCount !== undefined) {
        itemCount = params.itemCount;
      }

      return itemCount;
    };

    var selectItem = function selectItem(item, otherStateToSet, cb) {
      otherStateToSet = pickState(otherStateToSet);
      internalSetState(_objectSpread({
        isOpen: params.defaultIsOpen,
        highlightedIndex: params.defaultHighlightedIndex,
        selectedItem: item,
        inputValue: params.itemToString(item)
      }, otherStateToSet), cb);
    };

    var selectItemAtIndex = function selectItemAtIndex(itemIndex, otherStateToSet, cb) {
      var item = itemsRef.current[itemIndex];

      if (item == null) {
        return;
      }

      selectItem(item, otherStateToSet, cb);
    };

    var selectHighlightedItem = function selectHighlightedItem(otherStateToSet, cb) {
      return selectItemAtIndex(getState().highlightedIndex, otherStateToSet, cb);
    };

    var moveHighlightedIndex = function moveHighlightedIndex(amount, otherStateToSet) {
      var itemCount = getItemCount();

      if (itemCount > 0) {
        var nextHighlightedIndex = getNextWrappingIndex(amount, getState().highlightedIndex, itemCount);
        setHighlightedIndex(nextHighlightedIndex, otherStateToSet);
      }
    };

    var highlightFirstOrLastIndex = function highlightFirstOrLastIndex(event, first, otherStateToSet) {
      var itemsLastIndex = getItemCount() - 1;

      if (itemsLastIndex < 0 || !getState().isOpen) {
        return;
      }

      event.preventDefault();
      setHighlightedIndex(first ? 0 : itemsLastIndex, otherStateToSet);
    };

    var keyDownHandlers = {
      ArrowDown: function ArrowDown(event) {
        event.preventDefault();

        if (getState().isOpen) {
          var amount = event.shiftKey ? 5 : 1;
          moveHighlightedIndex(amount, {
            type: useDownshift.stateChangeTypes.keyDownArrowDown
          });
        } else {
          internalSetState({
            isOpen: true,
            type: useDownshift.stateChangeTypes.keyDownArrowDown
          }, function () {
            var itemCount = getItemCount();

            if (itemCount > 0) {
              setHighlightedIndex(getNextWrappingIndex(1, getState().highlightedIndex, itemCount), {
                isOpen: true,
                type: useDownshift.stateChangeTypes.keyDownArrowDown
              });
            }
          });
        }
      },
      ArrowUp: function ArrowUp(event) {
        event.preventDefault();

        if (getState().isOpen) {
          var amount = event.shiftKey ? -5 : -1;
          moveHighlightedIndex(amount, {
            type: useDownshift.stateChangeTypes.keyDownArrowUp
          });
        } else {
          internalSetState({
            isOpen: true,
            type: useDownshift.stateChangeTypes.keyDownArrowUp
          }, function () {
            var itemCount = getItemCount();

            if (itemCount > 0) {
              setHighlightedIndex(getNextWrappingIndex(-1, getState().highlightedIndex, itemCount), {
                isOpen: true,
                type: useDownshift.stateChangeTypes.keyDownArrowDown
              });
            }
          });
        }
      },
      Enter: function Enter(event) {
        var _getState = getState(),
            isOpen = _getState.isOpen,
            highlightedIndex = _getState.highlightedIndex;

        if (isOpen && highlightedIndex != null) {
          event.preventDefault();
          var item = itemsRef.current[highlightedIndex];
          var itemNode = getItemNodeFromIndex(highlightedIndex);

          if (item == null || itemNode && itemNode.hasAttribute('disabled')) {
            return;
          }

          selectHighlightedItem({
            type: useDownshift.stateChangeTypes.keyDownEnter
          });
        }
      },
      Escape: function Escape(event) {
        event.preventDefault();
        reset({
          type: useDownshift.stateChangeTypes.keyDownEscape,
          selectedItem: null,
          inputValue: ''
        });
      }
    };

    var inputKeyDownHandlers = _objectSpread({}, keyDownHandlers, {
      Home: function Home(event) {
        highlightFirstOrLastIndex(event, true, {
          type: useDownshift.stateChangeTypes.keyDownHome
        });
      },
      End: function End(event) {
        highlightFirstOrLastIndex(event, false, {
          type: useDownshift.stateChangeTypes.keyDownEnd
        });
      }
    });

    var buttonKeyDownHandlers = _objectSpread({}, keyDownHandlers, {
      ' ': function _(event) {
        event.preventDefault();
        toggleMenu({
          type: useDownshift.stateChangeTypes.keyDownSpaceButton
        });
      }
    });

    var inputHandleChange = function inputHandleChange(event) {
      internalSetState({
        type: useDownshift.stateChangeTypes.changeInput,
        isOpen: true,
        inputValue: event.target.value,
        highlightedIndex: params.defaultHighlightedIndex
      });
    };

    var inputHandleKeyDown = function inputHandleKeyDown(event) {
      var key = normalizeArrowKey(event);

      if (key && inputKeyDownHandlers[key]) {
        inputKeyDownHandlers[key](event);
      }
    };

    var setHighlightedIndex = function setHighlightedIndex() {
      var highlightedIndex = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : params.defaultHighlightedIndex;
      var otherStateToSet = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      otherStateToSet = pickState(otherStateToSet);
      internalSetState(_objectSpread({
        highlightedIndex: highlightedIndex
      }, otherStateToSet));
    };

    var inputHandleBlur = function inputHandleBlur() {
      internalSetTimeout(function () {
        var downshiftButtonIsActive = window.document && !!window.document.activeElement && !!window.document.activeElement.dataset && window.document.activeElement.dataset.toggle && rootRef.current && rootRef.current.contains(window.document.activeElement); // if (!this.isMouseDown && !downshiftButtonIsActive) {

        if (!downshiftButtonIsActive) {
          reset({
            type: useDownshift.stateChangeTypes.blurInput
          });
        }
      });
    };

    var buttonHandleClick = function buttonHandleClick(event) {
      event.preventDefault();

      if (window.document.activeElement === window.document.body) {
        event.target.focus();
      }

      internalSetTimeout(function () {
        return toggleMenu({
          type: useDownshift.stateChangeTypes.clickButton
        });
      });
    };

    var buttonHandleKeyDown = function buttonHandleKeyDown(event) {
      var key = normalizeArrowKey(event);

      if (buttonKeyDownHandlers[key]) {
        buttonKeyDownHandlers[key](event);
      }
    };

    var buttonHandleKeyUp = function buttonHandleKeyUp(event) {
      event.preventDefault();
    };

    var buttonHandleBlur = function buttonHandleBlur(event) {
      var blurTarget = event.target;
      internalSetTimeout(function () {
        if (!isMouseDownRef.current && (window.document.activeElement == null || window.document.activeElement.id !== params.inputId) && window.document.activeElement !== blurTarget) {
          reset({
            type: useDownshift.stateChangeTypes.blurButton
          });
        }
      });
    };

    var getRootProps = function getRootProps() {
      var _objectSpread2;

      var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          _ref4$refKey = _ref4.refKey,
          refKey = _ref4$refKey === void 0 ? 'ref' : _ref4$refKey,
          rest = _objectWithoutProperties(_ref4, ["refKey"]);

      var isOpen = state.isOpen;
      var _params = params,
          menuId = _params.menuId,
          labelId = _params.labelId;
      return _objectSpread((_objectSpread2 = {}, _defineProperty(_objectSpread2, refKey, rootRef), _defineProperty(_objectSpread2, "role", 'combobox'), _defineProperty(_objectSpread2, 'aria-expanded', isOpen), _defineProperty(_objectSpread2, 'aria-haspopup', 'listbox'), _defineProperty(_objectSpread2, 'aria-owns', isOpen ? menuId : null), _defineProperty(_objectSpread2, 'aria-labelledby', labelId), _objectSpread2), rest);
    };

    var getMenuProps = function getMenuProps() {
      var _objectSpread3;

      var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          _ref5$refKey = _ref5.refKey,
          refKey = _ref5$refKey === void 0 ? 'ref' : _ref5$refKey,
          ref = _ref5.ref,
          props = _objectWithoutProperties(_ref5, ["refKey", "ref"]);

      var _params2 = params,
          menuId = _params2.menuId,
          labelId = _params2.labelId;
      return _objectSpread((_objectSpread3 = {}, _defineProperty(_objectSpread3, refKey, callAll(ref, function (r) {
        return menuRef.current = r;
      })), _defineProperty(_objectSpread3, "role", 'listbox'), _defineProperty(_objectSpread3, 'aria-labelledby', props && props['aria-label'] ? null : labelId), _defineProperty(_objectSpread3, "id", menuId), _objectSpread3), props);
    };

    var getItemProps = function getItemProps(_ref6) {
      var onMouseMove = _ref6.onMouseMove,
          onMouseDown = _ref6.onMouseDown,
          onClick = _ref6.onClick,
          index = _ref6.index,
          item = _ref6.item,
          rest = _objectWithoutProperties(_ref6, ["onMouseMove", "onMouseDown", "onClick", "index", "item"]);

      if (index === undefined) {
        itemsRef.current.push(item);
        index = itemsRef.current.indexOf(item);
      } else {
        itemsRef.current[index] = item;
      }

      var enabledEventHandlers = {
        onMouseMove: callAllEventHandlers(onMouseMove, function () {
          if (index === getState().highlightedIndex) {
            return;
          }

          setHighlightedIndex(index, {
            type: useDownshift.stateChangeTypes.itemMouseEnter
          });
          avoidScrollingRef.current = true;
          internalSetTimeout(function () {
            return avoidScrollingRef.current = false;
          }, 250);
        }),
        onMouseDown: callAllEventHandlers(onMouseDown, function (event) {
          event.preventDefault();
        }),
        onClick: callAllEventHandlers(onClick, function () {
          selectItemAtIndex(index, {
            type: useDownshift.stateChangeTypes.clickItem
          });
        })
      };
      var eventHandlers = rest.disabled ? {
        onMouseDown: enabledEventHandlers.onMouseDown
      } : enabledEventHandlers;
      return _objectSpread({
        id: params.getItemId(index),
        role: 'option',
        'aria-selected': getState().highlightedIndex === index
      }, eventHandlers, rest);
    };

    var getLabelProps = function getLabelProps(props) {
      var _params3 = params,
          labelId = _params3.labelId,
          inputId = _params3.inputId;
      return _objectSpread({
        id: labelId,
        htmlFor: inputId
      }, props);
    };

    var getInputProps = function getInputProps() {
      var _ref7 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          onKeyDown = _ref7.onKeyDown,
          onBlur = _ref7.onBlur,
          onChange = _ref7.onChange,
          onInput = _ref7.onInput,
          onChangeText = _ref7.onChangeText,
          rest = _objectWithoutProperties(_ref7, ["onKeyDown", "onBlur", "onChange", "onInput", "onChangeText"]);

      var eventHandlers = {};
      var isOpen = state.isOpen,
          inputValue = state.inputValue,
          highlightedIndex = state.highlightedIndex;
      var _params4 = params,
          menuId = _params4.menuId,
          labelId = _params4.labelId,
          inputId = _params4.inputId,
          getItemId = _params4.getItemId;

      if (!rest.disabled) {
        eventHandlers = {
          onChange: callAllEventHandlers(onChange, onInput, inputHandleChange),
          onKeyDown: callAllEventHandlers(onKeyDown, inputHandleKeyDown),
          onBlur: callAllEventHandlers(onBlur, inputHandleBlur)
        };
      }

      return _objectSpread({
        'aria-autocomplete': 'list',
        'aria-activedescendant': isOpen && typeof highlightedIndex === 'number' && highlightedIndex >= 0 ? getItemId(highlightedIndex) : null,
        'aria-controls': isOpen ? menuId : null,
        'aria-labelledby': labelId,
        autoComplete: 'off',
        value: inputValue,
        id: inputId
      }, eventHandlers, rest);
    };

    var getToggleButtonProps = function getToggleButtonProps() {
      var _ref8 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          onClick = _ref8.onClick,
          onPress = _ref8.onPress,
          onKeyDown = _ref8.onKeyDown,
          onKeyUp = _ref8.onKeyUp,
          onBlur = _ref8.onBlur,
          rest = _objectWithoutProperties(_ref8, ["onClick", "onPress", "onKeyDown", "onKeyUp", "onBlur"]);

      var _getState2 = getState(),
          isOpen = _getState2.isOpen;

      var enabledEventHandlers = {
        onClick: callAllEventHandlers(onClick, buttonHandleClick),
        onKeyDown: callAllEventHandlers(onKeyDown, buttonHandleKeyDown),
        onKeyUp: callAllEventHandlers(onKeyUp, buttonHandleKeyUp),
        onBlur: callAllEventHandlers(onBlur, buttonHandleBlur)
      };
      var eventHandlers = rest.disabled ? {} : enabledEventHandlers;
      return _objectSpread({
        type: 'button',
        role: 'button',
        'aria-label': isOpen ? 'close menu' : 'open menu',
        'aria-haspopup': true,
        'data-toggle': true
      }, eventHandlers, rest);
    };

    var toggleMenu = function toggleMenu() {
      var otherStateToSet = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var cb = arguments.length > 1 ? arguments[1] : undefined;
      otherStateToSet = pickState(otherStateToSet);
      var newState = {};
      internalSetState(function (_ref9) {
        var isOpen = _ref9.isOpen;
        newState = _objectSpread({
          isOpen: !isOpen
        }, isOpen && {
          highlightedIndex: params.defaultHighlightedIndex
        }, otherStateToSet);
        return newState;
      }, function () {
        // TODO: review
        var _getState3 = getState(),
            isOpen = _getState3.isOpen,
            highlightedIndex = _getState3.highlightedIndex; // const { isOpen, highlightedIndex } = newState


        if (isOpen) {
          if (getItemCount() > 0 && typeof highlightedIndex === 'number') {
            setHighlightedIndex(highlightedIndex, otherStateToSet);
          }
        }

        cbToCb(cb)();
      });
    };

    var openMenu = function openMenu(cb) {
      internalSetState({
        isOpen: true
      }, cb);
    };

    var closeMenu = function closeMenu(cb) {
      internalSetState({
        isOpen: false
      }, cb);
    };

    var clearSelection = function clearSelection(cb) {
      internalSetState({
        selectedItem: null,
        inputValue: '',
        highlightedIndex: params.defaultHighlightedIndex,
        isOpen: params.defaultIsOpen
      }, cb);
    };

    var setItemCount = function setItemCount(count) {
      itemCountRef.current = count;
    };

    var unsetItemCount = function unsetItemCount() {
      itemCountRef.current = null;
    };

    var getStateAndHelpers = function getStateAndHelpers() {
      var isOpen = state.isOpen,
          inputValue = state.inputValue,
          selectedItem = state.selectedItem,
          highlightedIndex = state.highlightedIndex;
      var _params5 = params,
          id = _params5.id,
          itemToString = _params5.itemToString;
      return {
        // prop getters
        getRootProps: getRootProps,
        getMenuProps: getMenuProps,
        getItemProps: getItemProps,
        getLabelProps: getLabelProps,
        getInputProps: getInputProps,
        getToggleButtonProps: getToggleButtonProps,
        // actions
        reset: reset,
        openMenu: openMenu,
        closeMenu: closeMenu,
        toggleMenu: toggleMenu,
        selectItem: selectItem,
        selectItemAtIndex: selectItemAtIndex,
        selectHighlightedItem: selectHighlightedItem,
        setHighlightedIndex: setHighlightedIndex,
        clearSelection: clearSelection,
        clearItems: clearItems,
        setItemCount: setItemCount,
        unsetItemCount: unsetItemCount,
        setState: setState,
        // props
        itemToString: itemToString,
        // derived
        id: id,
        // state
        highlightedIndex: highlightedIndex,
        inputValue: inputValue,
        isOpen: isOpen,
        selectedItem: selectedItem
      };
    };

    clearItems();
    return getStateAndHelpers();
  }

  useDownshift.stateChangeTypes = {
    unknown: '__use-downshift_unknown__',
    mouseUp: '__use-downshift_mouseup__',
    itemMouseEnter: '__use-downshift_item_mouseenter__',
    keyDownArrowUp: '__use-downshift_keydown_arrow_up__',
    keyDownArrowDown: '__use-downshift_keydown_arrow_down__',
    keyDownEscape: '__use-downshift_keydown_escape__',
    keyDownEnter: '__use-downshift_keydown_enter__',
    keyDownHome: '__use-downshift_keydown_home__',
    keyDownEnd: '__use-downshift_keydown_end__',
    clickItem: '__use-downshift_click_item__',
    blurInput: '__use-downshift_blur_input__',
    changeInput: '__use-downshift_change_input__',
    keyDownSpaceButton: '__use-downshift_keydown_space_button__',
    clickButton: '__use-downshift_click_button__',
    blurButton: '__use-downshift_blur_button__',
    controlledPropUpdatedSelectedItem: '__use-downshift_controlled_prop_updated_selected_item__',
    touchEnd: '__use-downshift_touchend__'
  };

  return useDownshift;

}));
//# sourceMappingURL=use-downshift.umd.js.map
